ack_generator library

Classes

AckField
Overrides the inferred schema and/or presence for a class-first field.
AckInfer
Marks a top-level Ack schema for immutable model-class generation.
AckModel
Marks a hand-written class for Ack schema generation.
AckType
Marks a top-level Ack schema for frozen legacy extension-type generation.
Email
Adds .email() to an inferred string schema.
JsonKey
An annotation used to specify how a field is serialized.
Max
Adds .max(value) to an inferred numeric schema.
MaxItems
Adds .maxItems(count) to an inferred collection schema.
MaxLength
Adds .maxLength(length) to an inferred string schema.
Min
Adds .min(value) to an inferred numeric schema.
MinItems
Adds .minItems(count) to an inferred collection schema.
MinLength
Adds .minLength(length) to an inferred string schema.
MultipleOf
Adds .multipleOf(value) to an inferred numeric schema.
Negative
Adds .negative() to an inferred numeric schema.
NotEmpty
Adds .notEmpty() to an inferred string schema.
Pattern
Adds .matches(pattern) to an inferred string schema.
Positive
Adds .positive() to an inferred numeric schema.
UniqueItems
Adds .unique() to an inferred collection schema.

Enums

AckCaseStyle
JSON field-name styles supported by class-first Ack generation.
AckFieldPresence
Overrides inferred input-presence for a class-first field.
AckUnknownPropertyPolicy
How a class-first object treats properties that are not declared fields.

Functions

ackGenerator(BuilderOptions options) → Builder
Creates the frozen legacy .g.dart builder for @AckType.
ackModelBuilder(BuilderOptions options) → Builder
Creates the dedicated modern model part.
ackModelJsonBuilder(BuilderOptions options) → Builder
Creates the dedicated modern JSON part after .ack.dart exists.