AckFieldPresence enum

Overrides inferred input-presence for a class-first field.

inferred keeps constructor-based presence. required always requires the JSON key. optional is valid only when the constructor can accept a missing value, with a discriminator-specific exception for union branches.

Inheritance
Available extensions
Annotations
  • @Deprecated('Use @Optional() or @Required(). AckFieldPresence will be removed in 2.0.0.')

Values

inferred → const AckFieldPresence
required → const AckFieldPresence
optional → const AckFieldPresence

Properties

hashCode → int
The hash code for this object.
no setterinherited
index → int
A numeric identifier for the enumerated value.
no setterinherited
name → String

Available on Enum, provided by the EnumName extension

The name of the enum value.
no setter
runtimeType → Type
A representation of the runtime type of the object.
no setterinherited

Methods

noSuchMethod(Invocation invocation) → dynamic
Invoked when a nonexistent method or property is accessed.
inherited
toString() → String
A string representation of this object.
inherited

Operators

operator ==(Object other) → bool
The equality operator.
inherited

Constants

values → const List<AckFieldPresence>
A constant List of the values in this enum, in order of their declaration.