Pattern class final

Adds .matches(pattern) to an inferred string schema.

Annotations
  • @Target.new({TargetKind.field, TargetKind.parameter})

Constructors

Pattern(String pattern)
const

Properties

hashCode → int
The hash code for this object.
no setterinherited
pattern → String
final
runtimeType → Type
A representation of the runtime type of the object.
no setterinherited

Methods

noSuchMethod(Invocation invocation) → dynamic
Invoked when a nonexistent method or property is accessed.
inherited
toString() → String
A string representation of this object.
inherited

Operators

operator ==(Object other) → bool
The equality operator.
inherited