inference library
Shared schema inference for Ack generators.
Consumers supply name resolution for model types and source imports.
Classes
- AckSchemaInference
- Infers Ack schema expressions from resolved Dart types and annotations.
Shared schema inference for Ack generators.
Consumers supply name resolution for model types and source imports.